import java.awt.geom.Point2D;

import sk.upjs.jpaz2.*;


public class TurtleMover extends TickTimer {

    // turtle to be moved
    private Turtle movingTurtle = null;
    
    // time, when movement started
    private long startTime = 0;
    
    // location, where movement started
    private Point2D startPoint;
    
    // target location
    private Point2D endPoint;
    
    // total duration of the movement
    private long moveDuration;
    
    public boolean isMoving() {
        return (movingTurtle != null);
    }

    public void moveTo(Turtle turtle, double x, double y, long duration, long tickPeriod) {
	if (turtle == null)
	    return;
	
	if (duration == 0) {
	    turtle.setPosition(x, y);
	    return;
	}
	
	// set turtle that will be smoothly moved to given target location
	movingTurtle = turtle;
	
	// store information about initial position and time, and expected duration of move
	startTime = System.currentTimeMillis();
	startPoint = turtle.getPosition();
	endPoint = new Point2D.Double(x, y);
	moveDuration = duration; 

	// check whether tickPeriod is OK
	if (tickPeriod <= 0)
	    tickPeriod = 50;
	 	
	setTickPeriod(tickPeriod);
    }
    
    public void moveTo(Turtle turtle, double x, double y, long time) {
	moveTo(turtle, x, y, time, 50);
    }
    
    @Override
    protected void onTick() {
	if (movingTurtle == null)
	    return;
	
	long currentTime = System.currentTimeMillis();
	
	// manage location in the case, when time for movement is over
	if (currentTime >= startTime + moveDuration) {
	    movingTurtle.setPosition(endPoint);
	    movingTurtle = null;
	    setTickPeriod(0);
	    return;
	}
	
	// in all other cases
	double moveFraction = (currentTime - startTime)/(double)moveDuration;
	double dx = endPoint.getX() - startPoint.getX();
	double dy = endPoint.getY() - startPoint.getY();
	movingTurtle.setPosition(startPoint.getX() + moveFraction*dx, startPoint.getY() + moveFraction*dy);
    }
}
